summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorTomás Touceda <chiiph@leap.se>2013-08-19 17:50:53 -0300
committerTomás Touceda <chiiph@leap.se>2013-08-20 15:08:40 -0300
commit53bc81a863351cdf7bfb7a252bffdd8870e71742 (patch)
tree3f4fcddb2767de468866b5299f38dc5bfbd62da7
parent5ae019af6e29697dd43700c614e06298d719ff96 (diff)
Migrate mx to work with the new couchdb structure and views
-rw-r--r--changes/bug_new_couch_structure2
-rw-r--r--src/leap/mx/alias_resolver.py12
-rw-r--r--src/leap/mx/couchdbhelper.py51
-rw-r--r--src/leap/mx/mail_receiver.py49
4 files changed, 59 insertions, 55 deletions
diff --git a/changes/bug_new_couch_structure b/changes/bug_new_couch_structure
new file mode 100644
index 0000000..eea0ced
--- /dev/null
+++ b/changes/bug_new_couch_structure
@@ -0,0 +1,2 @@
+ o Migrate mx functions to work on the new couchdb structure and
+ views. Fixes #3502. \ No newline at end of file
diff --git a/src/leap/mx/alias_resolver.py b/src/leap/mx/alias_resolver.py
index 08ebb73..45a3ed2 100644
--- a/src/leap/mx/alias_resolver.py
+++ b/src/leap/mx/alias_resolver.py
@@ -80,16 +80,8 @@ class AliasResolverFactory(postfix.PostfixTCPMapDeferringDictServerFactory):
At some point we will have to consider the domain part too.
"""
try:
- log.msg("Processing key: %s" % (key,))
- if key.find("@") == -1:
- # No proper email address, but we need to continue processing
- # the query so postmap is happy.
- log.msg("Key it's not an email address")
- else:
- key = key.split("@")[0]
- key = key.split("+")[0]
- log.msg("Final key to query: %s" % (key,))
- d = self._cdb.queryByLoginOrAlias(key)
+ log.msg("Query key: %s" % (key,))
+ d = self._cdb.queryByAddress(key)
d.addCallback(self._to_str)
d.addCallback(self.spit_result)
diff --git a/src/leap/mx/mail_receiver.py b/src/leap/mx/mail_receiver.py
index a830fa1..ff2caa9 100644
--- a/src/leap/mx/mail_receiver.py
+++ b/src/leap/mx/mail_receiver.py
@@ -29,6 +29,7 @@ from email import message_from_string
from twisted.application.service import Service
from twisted.internet import inotify
+from twisted.internet.defer import DeferredList
from twisted.python import filepath, log
from leap.soledad.document import SoledadDocument
@@ -76,18 +77,20 @@ class MailReceiver(Service):
callbacks=[self._process_incoming_email],
recursive=recursive)
- def _get_pubkey(self, uuid):
- """
- Given a UUID for a user, retrieve its public key
+ def _gather_uuid_pubkey(self, results):
+ if len(results) < 2:
+ return None, None
- @param uuid: UUID for a user
- @type uuid: str
+ # DeferredList results are structured like this:
+ # [ (succeeded, pubkey), (succeeded, uuid) ]
+ # succeeded is a bool value that specifies if the
+ # corresponding callback succeeded
+ pubkey_res, uuid_res = results
- @return: uuid, public key
- @rtype: tuple of (str, str)
- """
- log.msg("Fetching pubkey for %s" % (uuid,))
- return uuid, self._users_cdb.getPubKey(uuid)
+ pubkey = pubkey_res[1] if pubkey_res[0] else None
+ uuid = uuid_res[1] if uuid_res[0] else None
+
+ return uuid, pubkey
def _encrypt_message(self, uuid_pubkey, address, message):
"""
@@ -128,14 +131,14 @@ class MailReceiver(Service):
key = gpg.list_keys().pop()
openpgp_key = openpgp._build_key_from_gpg(address, key, pubkey)
- doc.content = {
- "incoming": True,
- "_enc_scheme": EncryptionSchemes.PUBKEY,
- "_enc_json": str(gpg.encrypt(
- json.dumps(data),
- openpgp_key.fingerprint,
- symmetric=False))
- }
+ doc.content = {
+ "incoming": True,
+ "_enc_scheme": EncryptionSchemes.PUBKEY,
+ "_enc_json": str(gpg.encrypt(
+ json.dumps(data),
+ openpgp_key.fingerprint,
+ symmetric=False))
+ }
return uuid, doc
@@ -159,7 +162,6 @@ class MailReceiver(Service):
uuid = 0
db = CouchDatabase(self._mail_couch_url, "user-%s" % (uuid,))
- db.put_doc(doc)
log.msg("Done exporting")
@@ -208,17 +210,16 @@ class MailReceiver(Service):
if owner is None:
log.err("Malformed mail, neither To: nor "
"Delivered-To: field")
- owner = owner.split("@")[0]
- owner = owner.split("+")[0]
log.msg("Mail owner: %s" % (owner,))
log.msg("%s received a new mail" % (owner,))
- d = self._users_cdb.queryByLoginOrAlias(owner)
- d.addCallbacks(self._get_pubkey, log.err)
+ dpubk = self._users_cdb.getPubKey(owner)
+ duuid = self._users_cdb.queryByAddress(owner)
+ d = DeferredList([dpubk, duuid])
+ d.addCallbacks(self._gather_uuid_pubkey, log.err)
d.addCallbacks(self._encrypt_message, log.err,
(owner, mail_data))
d.addCallbacks(self._export_message, log.err)
d.addCallbacks(self._conditional_remove, log.err,
(filepath,))
d.addErrback(log.err)
-
diff --git a/src/leap/mx/couchdbhelper.py b/src/leap/mx/couchdbhelper.py
index 02ef088..147e6f9 100644
--- a/src/leap/mx/couchdbhelper.py
+++ b/src/leap/mx/couchdbhelper.py
@@ -97,7 +97,7 @@ class ConnectedCouchDB(client.CouchDB):
"""
pass
- def queryByLoginOrAlias(self, alias):
+ def queryByAddress(self, address):
"""
Check to see if a particular email or alias exists.
@@ -106,47 +106,56 @@ class ConnectedCouchDB(client.CouchDB):
@return: a deferred for this query
@rtype twisted.defer.Deferred
"""
- assert isinstance(alias, str), "Email or alias queries must be string"
+ assert isinstance(address, (str, unicode)), "Email or alias queries must be string"
# TODO: Cache results
- d = self.openView(docId="User",
- viewId="by_login_or_alias/",
- key=alias,
+ d = self.openView(docId="Identity",
+ viewId="by_address/",
+ key=address,
reduce=False,
include_docs=True)
- d.addCallbacks(partial(self._get_uuid, alias), log.err)
+ d.addCallbacks(partial(self._get_uuid, address), log.err)
return d
- def _get_uuid(self, alias, result):
+ def _get_uuid(self, address, result):
"""
- Parses the result of the by_login_or_alias query and gets the
- uuid
+ Parses the result of the by_address query and gets the uuid
- @param alias: alias looked up
- @type alias: string
+ @param address: alias looked up
+ @type address: string
@param result: result dictionary
@type result: dict
@return: The uuid for alias if available
@rtype: str
"""
for row in result["rows"]:
- if row["key"] == alias:
- uuid = row["id"]
- self._cache[uuid] = row["doc"].get("public_key", None)
+ if row["key"] == address:
+ uuid = row["doc"].get("user_id", None)
+ if uuid is None:
+ log.msg("ERROR: Found doc for %s but there's not user_id!"
+ % (address,))
return uuid
return None
- def getPubKey(self, uuid):
- pubkey = None
- try:
- pubkey = self._cache[uuid]
- except:
- pass
- return pubkey
+ def getPubKey(self, address):
+ d = self.openView(docId="Identity",
+ viewId="pgp_key_by_email/",
+ key=address,
+ reduce=False,
+ include_docs=True)
+ d.addCallbacks(partial(self._get_pgp_key, address), log.err)
+
+ return d
+
+ def _get_pgp_key(self, address, result):
+ for row in result["rows"]:
+ if row["key"] == address:
+ return row["value"]
+ return None
if __name__ == "__main__":
from twisted.internet import reactor